# -*- coding: utf-8 -*-
import gzip
import urllib2
import json
import re
from StringIO import StringIO
import xbmcaddon
import xbmc

acesrv = "http://%s/" % xbmcaddon.Addon().getSetting('aceserver')

user_agent = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.9.0.3) Gecko/2008092417 Firefox/3.0.3'
acelive_url = "http://content.asplaylist.net/cdn/{}.acelive&hlc=1&spv=0" \
              "&transcode_audio=0&transcode_mp3=0&transcode_ac3=0&preferred_audio_language=eng"
acestream_url = 'http://ace-stream.tv/as.json'
tuchkatv_url = 'http://tuchkatv.org/player.html'
acetv_url = 'http://acetv.org/js/data.json'


def log(*s):
    if xbmcaddon.Addon().getSetting('debug') == 'true':
        xbmc.log(str(u'TVGyvai ' + u': ' + u': '.join([unicode(x) for x in s])), xbmc.LOGNOTICE)


def download(url, ret_json=False):
    req = urllib2.Request(url)
    log('Downloading', url)
    req.add_header('User-Agent', user_agent)
    response = urllib2.urlopen(req, timeout=120)
    if response.info().get('Content-Encoding') == 'gzip':
        buf = StringIO(response.read())
        f = gzip.GzipFile(fileobj=buf)
        data = f.read()
    else:
        data = response.read()
    response.close()
    encoding = response.info().get('Content-Type').split('charset=')[-1]
    if encoding:
        try:
            data = unicode(data, encoding)
        except LookupError:
            pass
    if ret_json:
        try:
            data = json.loads(data)
        except ValueError:
            return {}
    return data


class Menu:
    def __init__(self, name, items=None, icon=None):
        self.label = name
        self.icon = icon
        self.image = None
        self.items = items or []


class Source:
    def __init__(self, name, link, icon=None):
        self.label = name
        # self.link = urllib.quote(link)
        self.link = link
        self.icon = icon


class AceUrlVideo(Source):
    def __init__(self, name, link, icon=None):
        Source.__init__(self, name, acesrv + 'ace/getstream?url=' + link, icon)


class AceIdVideo(Source):
    def __init__(self, name, link, icon=None):
        self.acelink = link
        Source.__init__(self, name, acesrv + 'ace/manifest.m3u8?id=' + link, icon)
        # self.getinfotag()

    def getinfotag(self):
        info = download(acesrv + 'ace/manifest.m3u8?id=' + self.acelink + '&format=json', ret_json=True)
        stat_url = info.get(u'response', {}).get(u'stat_url')
        log('AceIdVideo', 'info ' + self.acelink, str(info))
        stat = download(stat_url + '?format=json', ret_json=True)
        log('AceIdVideo', 'stat ' + self.acelink, str(stat))
        peers = stat.get(u'response', {}).get(u'peers', -1)
        if peers >= 0:
            self.label += ' [COLOR red][%d žiuri][/COLOR]' % peers
        else:
            self.label += ' [COLOR red][Aktyvus][/COLOR]'


class AceLiveVideo(AceUrlVideo):
    def __init__(self, name, link, icon=None):
        AceUrlVideo.__init__(self, name, acelive_url.format(link), icon)


def load_acestreamtv():
    return load_channel_json(acestream_url)


def load_acetv():
    return load_channel_json(acetv_url)


def load_channel_json(url):
    data = download(url)
    channels = json.loads(data)
    res = []
    chs = {}
    for ch in channels[u'channels']:
        cat = ch[u'cat'].lower().replace('_', ' ').capitalize()
        if cat in chs:
            chs[cat].append(ch)
        else:
            chs[cat] = [ch]
    for n, chl in chs.items():
        res.append(Menu(n, sorted([
            AceIdVideo(ch[u'name'], ch[u'url']) for ch in chl
        ], key=lambda x: x.label)))
    res = sorted(res, key=lambda x: x.label)
    res = [Menu(u'Visi', [AceIdVideo(ch[u'name'], ch[u'url']) for ch in channels[u'channels']])] + res

    return res


def load_tuchkatv():
    data = download(tuchkatv_url)
    channels = re.findall(r'<option.+value="([abcdef0-9]{40})".*>(.+)<\/option>', data, re.IGNORECASE)
    res = []
    for url, name in channels:
        name = name.replace(u'</a>', u'')
        res.append(AceIdVideo(name, url))
    return res


def lttv():
    ret = []
    # LRT
    data = download('https://www.lrt.lt/servisai/stream_url/live/get_live_url.php?channel=LTV1', ret_json=True)
    content = data.get('response', {}).get('data', {}).get('content')
    if content: ret.append(Source("LRT Televizija", content))

    data = download('https://www.lrt.lt/servisai/stream_url/live/get_live_url.php?channel=LTV2', ret_json=True)
    content = data.get('response', {}).get('data', {}).get('content')
    if content: ret.append(Source("LRT Plius", content))

    data = download('https://www.lrt.lt/servisai/stream_url/live/get_live_url.php?channel=WORLD', ret_json=True)
    content = data.get('response', {}).get('data', {}).get('content')
    if content: ret.append(Source("LRT Lituanica", content))

    data = download('https://www.lrt.lt/servisai/stream_url/live/get_live_url.php?channel=LR', ret_json=True)
    content = data.get('response', {}).get('data', {}).get('content')
    if content: ret.append(Source("LRT Radijas", content))

    data = download('https://www.lrt.lt/servisai/stream_url/live/get_live_url.php?channel=Klasika', ret_json=True)
    content = data.get('response', {}).get('data', {}).get('content2')
    if content: ret.append(Source("LRT Klasika", content))

    data = download('https://www.lrt.lt/servisai/stream_url/live/get_live_url.php?channel=Opus', ret_json=True)
    content = data.get('response', {}).get('data', {}).get('content')
    if content: ret.append(Source("LRT Opus", content))

    # internetine-tv.narod.ru
    # data = download('http://www.internetine-tv.narod.ru/awv38koo4ffO07.html')
    # channels = re.findall(r'<option.+value=\".*([abcdef0-9]{40})\?.*\".*>(.+)<\/option>', data, re.IGNORECASE)
    # for url, name in channels:
    #     ret.append(AceIdVideo(name, url))

    # infotv
    data = download('https://lnk.lt/api/main/video-page/info-tv-hd-kanalas-internetu-100/62591/false', ret_json=True)
    info = data.get('videoConfig', {}).get('videoInfo', {})
    url = info.get('videoUrl', '') + info.get('secureTokenParams', '')
    if url: ret.append(Source("INFO TV", url))

    # Static
    ret.append(AceIdVideo("LNK", "74e7f37525aa99fd42e5e04c3f7187c59bf31c9e"))
    return ret

main_menu = [
    Menu("Lietuviška TV", lttv),
    Menu("Ace-stream.tv", load_acestreamtv),
    Menu("Ace TV", load_acetv),
    Menu("Tuchka TV", load_tuchkatv),
]
